// Copyright 2022 susatthi All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:roggle/roggle.dart';
import 'package:universal_platform/universal_platform.dart';

final logger = Roggle(
  printer: SinglePrettyPrinter(
    loggerName: ' [APP]',
    // warning 以上のときはスタックトレースを出力する
    stackTraceLevel: Level.warning,
    // iOS はカラー非対応
    colors: !UniversalPlatform.isIOS,
    // ログが長くなるので非表示
    printCaller: false,
  ),
  output: _AssertionOutput(),
);

class _AssertionOutput extends ConsoleOutput {
  @override
  void output(OutputEvent event) {
    super.output(event);

    // wtf 以上のときは処理を停止する
    if (event.level.index >= Level.wtf.index) {
      throw AssertionError('Stopped by logger.');
    }
  }
}

class ProviderLogger extends ProviderObserver {
  @override
  void didAddProvider(
    ProviderBase<dynamic> provider,
    Object? value,
    ProviderContainer container,
  ) {
    final name = provider.name;
    if (name != null) {
      logger.v(
        '[RIVERPOD][ADD] $name: value = $value',
      );
    }
  }

  @override
  void providerDidFail(
    ProviderBase<dynamic> provider,
    Object error,
    StackTrace stackTrace,
    ProviderContainer container,
  ) {
    final name = provider.name;
    if (name != null) {
      logger.w(
        '[RIVERPOD][ERROR] $name',
        error,
        stackTrace,
      );
    }
  }

  @override
  void didUpdateProvider(
    ProviderBase<dynamic> provider,
    Object? previousValue,
    Object? newValue,
    ProviderContainer container,
  ) {
    final name = provider.name;
    if (name != null) {
      logger.v(
        '[RIVERPOD][UPDATE] $name: '
        'newValue = $newValue',
      );
    }
  }

  @override
  void didDisposeProvider(
    ProviderBase<dynamic> provider,
    ProviderContainer container,
  ) {
    final name = provider.name;
    if (name != null) {
      logger.v(
        '[RIVERPOD][DISPOSE] $name',
      );
    }
  }
}
